#ifndef _LIBLVE_KAPI_H_
#define _LIBLVE_KAPI_H_

#include <stdint.h>

#define LVE_DEV		"/dev/"LVE_DEV_NAME
#define LVE_API_VERSION(a,b) (((a) << 16) + (b))

struct liblve;

/** userland -> kernel api translation */
struct kapi {
	/**
	  is that version api is applicable
	  @param strong need strong api version check

	  @retval 0  api applicable
	  @retval <0 error
	*/
	int (*lib_check)(int strong);
	/** 
	  initialize api instance
	 */
	int (*lib_init)(struct liblve *buffer);
	/**
	 destroy api instance
	 */
	int (*lib_fini)(struct liblve *buffer);
	/* */
	int (*lve_enter)(struct liblve *lib, uint32_t veid, uint32_t *cookie);
	int (*lve_leave)(struct liblve *lib, uint32_t *cookie);
	/* admin rights */
	int (*lve_default)(int32_t cpu, int32_t io, int32_t enter);
	int (*lve_create)(uint32_t veid);
	int (*lve_destroy)(uint32_t veid);
	int (*lve_setup)(uint32_t veid, int32_t cpu, int32_t io, int32_t enters);
	int (*lve_setup_enter)(uint32_t veid, int32_t cpu, int32_t io, int32_t enters, uint32_t *cookie);
	int (*lve_flush)(int all);
	int (*lve_info)(uint32_t veid, int32_t *cpu, int32_t *io, int32_t *enters);
};

extern struct kapi	*kern;

#define KAPI_CALL(name, ...) { 				\
				if (kern == NULL)	\
					return -ENOSYS;	\
				if (kern->name == NULL) \
					return -ENOSYS;	\
				else			\
					return kern->name(__VA_ARGS__); \
			    } while (0);

extern struct kapi	api_04;
extern struct kapi	api_05;

int kapi_init(void);

#endif
