#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <grp.h>
#include <pwd.h>

#include "kapi.h"
#include "lve-api-0.4.h"

struct liblve {
	int fd;
};

static int
lve_cmd_adm(int cmd, uint32_t id, void *data)
{
	int fd, rc;
	struct ve_ioctl vectl;
	uint32_t api_ver;

	fd = open(LVE_DEV, O_RDONLY);
	if (fd == -1)
		return -errno;

	vectl.id = id;
	vectl.data = data;

	rc = ioctl(fd, cmd, &vectl);
	close(fd);

	return rc;
}

static int
lve_cmd(struct liblve *lib, int cmd, uint32_t ve_id, void *data)
{
	int rc;
	struct ve_ioctl vectl;

	vectl.id = ve_id;
	vectl.data = data;

	rc = ioctl(lib->fd, cmd, &vectl);

	return rc;
}

static int api04_check(int strong)
{
	int fd, rc;
	struct ve_ioctl vectl;
	int32_t api_ver;

	/* strong check = check version */
	api_ver = lve_cmd_adm(API_VER, 0, NULL);
	if (api_ver < 0)
		return 0;

	if (strong)
		rc = (api_ver == LVE_API_VERSION(LVE_KMOD_API_MAJOR, LVE_KMOD_API_MINOR) ||
		      api_ver == LVE_API_VERSION(LVE_KMOD_API_MAJOR2, LVE_KMOD_API_MINOR2));
	else
		rc = api_ver > LVE_API_VERSION(LVE_KMOD_API_MAJOR2, LVE_KMOD_API_MINOR2);

	return  rc;
}

static int api04_init(struct liblve *lve)
{
	if (lve == NULL)
		return sizeof(struct liblve);

	lve->fd = open(LVE_DEV, O_RDONLY);
	if (lve->fd == -1)
		return -errno;

	return 0;
}

static int api04_fini(struct liblve *lve)
{
	if (lve == NULL)
		return -EINVAL;

	close(lve->fd);

	return 0;
}

/*****************************************************/
static int
api04_enter(struct liblve *lib,
	  uint32_t ve_id, uint32_t *cookie)
{
	struct ve_enter cmd;
	int rc;

	if (cookie == NULL)
		return -EINVAL;

	cmd.uid = -1;
	cmd.gid = -1;
	cmd.cookie = cookie;
	/* XXX */
	cmd.grp_size = -1;
	
	rc = lve_cmd(lib, ENTER_VE, ve_id, &cmd);

	return rc;
}

static int api04_leave(struct liblve *lib, uint32_t *cookie)
{
	struct ve_leave cmd;

	if (cookie == NULL)
		return -EINVAL;

	cmd.cookie = cookie;
	return lve_cmd(lib, LEAVE_VE, 0, &cmd);
}

/************************************************************/

static int
api04_set_default(int32_t cpu, int32_t io, int32_t maxentry)
{
	struct ve_config cfg;
	unsigned cpulim1024 = (float)cpu * 1024 / 100;

	cfg.cpu = cpulim1024;
	cfg.io = io;
	cfg.enters = maxentry;

	return lve_cmd_adm(DEFAULT_PARAMS, 0, &cfg);
}

static int api04_create(uint32_t ve_id)
{
	return lve_cmd_adm(CREATE_VE, ve_id, NULL);
}

int api04_destroy(uint32_t ve_id)
{
	return lve_cmd_adm(DESTROY_VE, ve_id, NULL);
}

static int api04_setup(uint32_t ve_id, 
		int32_t cpu, int32_t io, int32_t maxentry)
{
	struct ve_config cfg;
	unsigned cpulim1024 = (float)cpu * 1024 / 100;

	cfg.cpu = cpulim1024;
	cfg.io = io;
	cfg.enters = maxentry;

	return lve_cmd_adm(SETUP_VE, ve_id, &cfg);
}

static int api04_setup_enter(uint32_t veid, 
		    int32_t cpu, int32_t io, int32_t maxentryproc,
		    uint32_t *cookie)
{
	int rc;

	return -ENOSYS;

	rc = api04_create(veid);
	if (rc < 0)
		return rc;

	rc = api04_setup(veid, cpu, io, maxentryproc);
	if (rc < 0)
		return rc;

	return 0;
}

static int api04_flush(int all)
{
	struct lve_flush cfg;

	cfg.all = all;
	return lve_cmd_adm(FLUSH_VE, 0, &cfg);
}

static int api04_info(uint32_t ve_id, 
		uint32_t *cpu, uint32_t *io, int32_t *maxentry)
{
	struct ve_config cfg;
	int rc;

	rc = lve_cmd_adm(INFO_VE, ve_id, &cfg);
	if (rc == 0) {
		*cpu = (cfg.cpu * 100) / 1024;
		*io = cfg.io;
		*maxentry = cfg.enters;
	}
	return rc;
}


struct kapi api_04 = {
	.lib_check	= api04_check,
	.lib_init	= api04_init,
	.lib_fini	= api04_fini,

	.lve_enter	= api04_enter,
	.lve_leave	= api04_leave,
	
	.lve_default	= api04_set_default,
	.lve_create	= api04_create,
	.lve_destroy	= api04_destroy,
	.lve_setup	= api04_setup,
	.lve_setup_enter = api04_setup_enter,
	.lve_flush	= api04_flush,
	.lve_info	= api04_info,
};

