#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <grp.h>
#include <pwd.h>

#include "kapi.h"
#include "lve-api-0.5.h"

struct liblve {
	int fd;
};

static int
lve_cmd_adm(int cmd, uint32_t id, void *data)
{
	int fd, rc;
	struct ve_ioctl vectl;

	fd = open(LVE_DEV, O_RDONLY);
	if (fd == -1)
		return -errno;

	vectl.id = id;
	vectl.data = data;

	rc = ioctl(fd, cmd, &vectl);
	close(fd);

	return rc;
}

static int
lve_cmd(struct liblve *lib, int cmd, uint32_t ve_id, void *data)
{
	int rc;
	struct ve_ioctl vectl;

	vectl.id = ve_id;
	vectl.data = data;

	rc = ioctl(lib->fd, cmd, &vectl);

	return rc;
}

static int api05_check(int strong)
{
	int fd, rc;
	struct ve_ioctl vectl;
	int32_t api_ver;

	/* strong check = check version */
	api_ver = lve_cmd_adm(API_VER, 0, &api_ver);
	if (rc < 0)
		return 0;

	if (strong)
		rc = api_ver == LVE_API_VERSION(LVE_KMOD_API_MAJOR, LVE_KMOD_API_MINOR);
	else
		rc = api_ver > LVE_API_VERSION(LVE_KMOD_API_MAJOR, LVE_KMOD_API_MINOR);

	return  rc;
}

static int api05_init(struct liblve *lve)
{
	if (lve == NULL)
		return sizeof(struct liblve);

	lve->fd = open(LVE_DEV, O_RDONLY);
	if (lve->fd == -1)
		return -errno;

	return 0;
}

static int api05_fini(struct liblve *lve)
{
	if (lve == NULL)
		return -EINVAL;

	close(lve->fd);

	return 0;
}

static int api05_enter(struct liblve *lve,
              uint32_t ve_id, uint32_t *cookie)
{
	struct ve_enter cmd;
	int rc;

	if (cookie == NULL)
		return -EINVAL;

	cmd.cookie = cookie;
	/* XXX */
	cmd.grp_size = -1;

	rc = lve_cmd(lve, ENTER_VE, ve_id, &cmd);
	return rc;
}

static int api05_leave(struct liblve *lib, uint32_t *cookie)
{
	struct ve_leave cmd;

	if (cookie == NULL)
		return -EINVAL;

	cmd.cookie = cookie;
	return lve_cmd(lib, LEAVE_VE, 0, &cmd);
}

static int
api05_set_default(int32_t cpu, int32_t io, int32_t maxentryproc)
{
	struct ve_config_05 cfg;

	cfg.ulimits[LIM_CPU] = cpu;
	cfg.ulimits[LIM_IO] = io;
	cfg.ulimits[LIM_ENTER] = maxentryproc;

	return lve_cmd_adm(DEFAULT_PARAMS_05, 0, &cfg);
}

static int api05_create(uint32_t ve_id)
{
	return lve_cmd_adm(CREATE_VE, ve_id, NULL);
}

static int api05_destroy(uint32_t ve_id)
{
	return lve_cmd_adm(DESTROY_VE, ve_id, NULL);
}

static int api05_setup(uint32_t ve_id, int32_t cpu, int32_t io, int32_t maxentryproc)
{
	struct ve_config_05 cfg;

	cfg.ulimits[LIM_CPU] = cpu;
	cfg.ulimits[LIM_IO] = io;
	cfg.ulimits[LIM_ENTER] = maxentryproc;

	return lve_cmd_adm(SETUP_VE_05, ve_id, &cfg);
}

static int api05_setup_enter(uint32_t ve_id, 
		    int32_t cpu, int32_t io, int32_t maxentryproc,
		    uint32_t *cookie)
{
	struct ve_enter  enter;
	struct ve_config_05 cfg;
	struct lve_setup_enter_05 param;

	enter.cookie = cookie;
	enter.grp_size = -1;

	cfg.ulimits[LIM_CPU] = cpu;
	cfg.ulimits[LIM_IO] = io;
	cfg.ulimits[LIM_ENTER] = maxentryproc;

	param.enter = &enter;
	param.setup = &cfg;

	return lve_cmd_adm(SETUP_ENTER_VE_05, ve_id, &param);
}

static int api05_flush(int all)
{
	struct lve_flush cfg;

	cfg.all = all;
	return lve_cmd_adm(FLUSH_VE, 0, &cfg);
}

static int api05_info(uint32_t ve_id, 
		int32_t *cpu, int32_t *io, int32_t *maxenters)
{
	struct ve_config_05 cfg;
	int rc;

	rc = lve_cmd_adm(INFO_VE_05, ve_id, &cfg);
	if (rc == 0) {
		*cpu = cfg.ulimits[LIM_CPU];
		*io = cfg.ulimits[LIM_IO];
		*maxenters = cfg.ulimits[LIM_ENTER];
	}
	return rc;
}

struct kapi api_05 = {
	.lib_check	= api05_check,
	.lib_init	= api05_init,
	.lib_fini	= api05_fini,

	.lve_enter	= api05_enter,
	.lve_leave	= api05_leave,

	.lve_default	= api05_set_default,
	.lve_create	= api05_create,
	.lve_destroy	= api05_destroy,
	.lve_setup	= api05_setup,
	.lve_setup_enter = api05_setup_enter,
	.lve_flush	= api05_flush,
	.lve_info	= api05_info,
};

